@@ -1,5 +1,5 @@
-# function depends on phytools (& dependencies) and maps (& dependencies)
-# written by Liam J. Revell 2013
+## function depends on phytools (& dependencies) and maps (& dependencies)
+## written by Liam J. Revell 2013, 2017, 2019
 
 phylo.to.map<-function(tree,coords,rotate=TRUE,...){
 	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
@@ -19,15 +19,20 @@ phylo.to.map<-function(tree,coords,rotate=TRUE,...){
 	# if rotate
 	if(hasArg(type)) type<-list(...)$type else type<-"phylogram"
 	if(hasArg(direction)) direction<-list(...)$direction else direction<-"downwards"
-	if(rotate&&type=="phylogram") tree<-minRotate(tree,coords[,if(direction=="rightwards") 1 else 2])
-	x<-list(tree=tree,map=map,coords=coords)
+	if(is.data.frame(coords)) coords<-as.matrix(coords)
+	if(rotate&&type=="phylogram"){
+		cc<-aggregate(coords,by=list(rownames(coords)),mean)
+		cc<-matrix(as.matrix(cc[,2:3]),nrow(cc),2,dimnames=list(cc[,1],colnames(cc)[2:3]))
+		tree<-minRotate(tree,cc[,if(direction=="rightwards") 1 else 2])
+	} else direction<-"unoptimized"
+	x<-list(tree=tree,map=map,coords=coords,direction=direction)
 	class(x)<-"phylo.to.map"
 	if(plot) plot.phylo.to.map(x,...)
 	invisible(x)
 }
 
-# function to plot object of class "phylo.to.map"
-# written by Liam J. Revell 2013, 2014, 2016
+## S3 method to plot object of class "phylo.to.map"
+## written by Liam J. Revell 2013, 2014, 2016, 2019, 2020
 
 plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 	type<-type[1]
@@ -67,6 +72,10 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 	if(length(colors)==2&&type=="phylogram"){
 		colors<-matrix(rep(colors,nrow(coords)),nrow(coords),2,byrow=TRUE)
 		rownames(colors)<-rownames(coords)
+	} else if(is.vector(colors)&&(length(colors)==Ntip(tree))) {
+		COLS<-matrix("red",nrow(coords),2,dimnames=list(rownames(coords)))
+		for(i in 1:length(colors)) COLS[which(rownames(COLS)==names(colors)[i]),1:2]<-colors[i]
+		colors<-COLS
 	}
 	if(hasArg(direction)) direction<-list(...)$direction
 	else direction<-"downwards"
@@ -80,6 +89,23 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 	else lty<-"dashed"
 	if(hasArg(pts)) pts<-list(...)$pts
 	else pts<-TRUE
+	if(hasArg(col.edge)) col.edge<-list(...)$col.edge
+	else col.edge<-rep(par()$fg,nrow(x$tree$edge))
+	if(type=="phylogram"){
+		if(x$direction=="downwards"&&direction=="rightwards"){
+			cat("\"phylo.to.map\" direction is \"downwards\" but plot direction has been given as \"rightwards\".\n")
+			cat("Re-optimizing object....\n")
+			cc<-aggregate(coords,by=list(rownames(coords)),mean)
+			cc<-matrix(as.matrix(cc[,2:3]),nrow(cc),2,dimnames=list(cc[,1],colnames(cc)[2:3]))
+			tree<-minRotate(tree,cc[,1])
+		} else if(x$direction=="rightwards"&&direction=="downwards"){
+			cat("\"phylo.to.map\" direction is \"rightwards\" but plot direction has been given as \"downwards\".\n")
+			cat("Re-optimizing object....\n")
+			cc<-aggregate(coords,by=list(rownames(coords)),mean)
+			cc<-matrix(as.matrix(cc[,2:3]),nrow(cc),2,dimnames=list(cc[,1],colnames(cc)[2:3]))
+			tree<-minRotate(tree,cc[,2])
+		}
+	}
 	# recompute ylim or xlim to leave space for the tree
 	if(type=="phylogram"){
 		if(direction=="downwards"){
@@ -92,13 +118,15 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 		}
 	}
 	# open & size a new plot
-	par(mar=mar)
+	if(all(mar==0)) mar<-mar+0.01
 	plot.new()
+	par(mar=mar)
 	plot.window(xlim=xlim,ylim=ylim,asp=asp)
 	map(map,add=TRUE,fill=TRUE,col="gray95",mar=rep(0,4))
 	if(type=="phylogram"){
 		## preliminaries
 		cw<-reorder(tree,"cladewise")
+		if(!is.binary(cw)) cw<-multi2di(cw)
 		n<-Ntip(cw)
 		if(direction=="downwards"){
 			# plot a white rectangle
@@ -125,19 +153,22 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 			# compute start & end points of each edge
 			Y<-ylim[2]-nodeHeights(cw)
 			# plot coordinates & linking lines
-			coords<-coords[cw$tip.label,2:1]
-			for(i in 1:n) lines(c(x[i],coords[i,1]),
-				c(Y[which(cw$edge[,2]==i),2]-
-					if(from.tip) 0 else sh[i],coords[i,2]),
-				col=colors[cw$tip.label,][i,1],lty=lty,lwd=lwd[2])
-			points(coords,pch=pch,cex=cex.points[2],bg=colors[cw$tip.label,2])
+			coords<-coords[,2:1]
+			for(i in 1:nrow(coords)){ 
+				tip.i<-which(cw$tip.label==rownames(coords)[i])
+				lines(c(x[tip.i],coords[i,1]),c(Y[which(cw$edge[,2]==tip.i),2]-
+					if(from.tip) 0 else sh[tip.i],coords[i,2]),
+					col=colors[i,1],lty=lty,lwd=lwd[2])
+			}
+			points(coords,pch=pch,cex=cex.points[2],bg=colors[,2])
 			# plot vertical edges
 			for(i in 1:nrow(Y)) lines(rep(x[cw$edge[i,2]],2),Y[i,],
 				lwd=lwd[1],lend=2)
 			# plot horizontal relationships
-			for(i in 1:cw$Nnode+n) 
-				lines(range(x[cw$edge[which(cw$edge[,1]==i),2]]),
-				Y[which(cw$edge[,1]==i),1],lwd=lwd[1],lend=2)
+			for(i in 1:cw$Nnode+n){
+				ii<-which(cw$edge[,1]==i)
+				if(length(ii)>1) lines(range(x[cw$edge[ii,2]]),Y[ii,1],lwd=lwd[1],lend=2)
+			}
 			# plot tip labels
 			for(i in 1:n){ 
 				if(ftype) text(x[i],Y[which(cw$edge[,2]==i),2],
@@ -175,14 +206,18 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 			}
 			H<-nodeHeights(cw)
 			X<-xlim[1]+H
-			coords<-coords[cw$tip.label,2:1]
-			for(i in 1:n) lines(c(X[which(cw$edge[,2]==i),2]+
-				if(from.tip) 0 else sh[i],coords[i,1]),
-				c(y[i],coords[i,2]),col=colors[cw$tip.label,][i,1],lty=lty,lwd=lwd[2])
-			points(coords,pch=pch,cex=cex.points[2],bg=colors[cw$tip.label,2])
+			coords<-coords[,2:1]
+			for(i in 1:nrow(coords)){
+				tip.i<-which(cw$tip.label==rownames(coords)[i])
+				lines(c(X[which(cw$edge[,2]==tip.i),2]+if(from.tip) 0 else sh[tip.i],coords[i,1]),
+					c(y[tip.i],coords[i,2]),col=colors[i,1],lty=lty,lwd=lwd[2])
+			}
+			points(coords,pch=pch,cex=cex.points[2],bg=colors[,2])
 			for(i in 1:nrow(X)) lines(X[i,],rep(y[cw$edge[i,2]],2),lwd=lwd[1],lend=2)
-			for(i in 1:cw$Nnode+n) lines(X[which(cw$edge[,1]==i),1],
-				range(y[cw$edge[which(cw$edge[,1]==i),2]]),lwd=lwd[1],lend=2)
+			for(i in 1:cw$Nnode+n){ 
+				ii<-which(cw$edge[,1]==i)
+				if(length(ii)>1) lines(X[ii,1],range(y[cw$edge[ii,2]]),lwd=lwd[1],lend=2)
+			}
 			for(i in 1:n){
 				if(ftype) text(X[which(cw$edge[,2]==i),2],y[i],
 					paste(" ",sub("_"," ",cw$tip.label[i]),sep=""),
@@ -210,7 +245,7 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 
 
 ## rotates all nodes to try and match tip an ordering 
-## written by Liam J. Revell 2013, 2015
+## written by Liam J. Revell 2013, 2015, 2020
 minRotate<-function(tree,x,...){
 	if(hasArg(print)) print<-list(...)$print
 	else print<-TRUE
@@ -218,7 +253,7 @@ minRotate<-function(tree,x,...){
 	nn<-1:tree$Nnode+Ntip(tree)
 	x<-x[tree$tip.label]
 	for(i in 1:tree$Nnode){
-		tt<-read.tree(text=write.tree(rotate(tree,nn[i])))
+		tt<-read.tree(text=write.tree(if(isSingleton(tree,nn[i])) tree else rotate(tree,nn[i])))
 		oo<-sum(abs(order(x[tree$tip.label])-1:length(tree$tip.label)))
 		pp<-sum(abs(order(x[tt$tip.label])-1:length(tt$tip.label)))
 		if(oo>pp) tree<-tt
@@ -228,11 +263,19 @@ minRotate<-function(tree,x,...){
 	return(tree)
 }
 
+isSingleton<-function(tree,node) if(sum(tree$edge[,1]==node)<=1) TRUE else FALSE
+
 print.phylo.to.map<-function(x,...){
 	cat("Object of class \"phylo.to.map\" containing:\n\n")
 	cat(paste("(1) A phylogenetic tree with",Ntip(x$tree),"tips and",x$tree$Nnode,"internal nodes.\n\n",sep=" "))
 	cat("(2) A geographic map with range:\n")
 	cat(paste("     ",paste(round(x$map$range[3:4],2),collapse="N, "),"N\n",sep=""))
 	cat(paste("     ",paste(round(x$map$range[1:2],2),collapse="W, "),"W.\n\n",sep=""))
-	cat(paste("(3) A table containing",nrow(x$coords),"geographic coordinates.\n\n"))
+	cat(paste("(3) A table containing ",nrow(x$coords)," geographic coordinates (may include\n",
+			"    more than one set per species).\n\n",sep=""))
+	if(x$direction%in%c("downwards","rightwards")) 
+		cat(paste("If optimized, tree nodes have been rotated to maximize alignment\n",
+			"with the map when the tree is plotted in a ",x$direction," direction.\n\n",sep=""))
+	else
+		cat("The nodes of the tree may not have yet been rotated to maximize\nalignment between the phylogeny & the map.\n\n")
 }
